@@ -185,12 +185,17 @@ mkEntityDef :: PersistSettings
             -> EntityDef
 mkEntityDef ps name entattribs lines =
     EntityDef
-        (HaskellName name)
-        (DBName $ psToDBName ps name)
+        (HaskellName name')
+        (DBName $ getDbName ps name' entattribs)
         (DBName $ idName entattribs)
         entattribs cols uniqs derives
         extras
+        isSum
   where
+    (isSum, name') =
+        case T.uncons name of
+            Just ('+', x) -> (True, x)
+            _ -> (False, name)
     (attribs, extras) = splitExtras lines
     idName [] = "id"
     idName (t:ts) =
@@ -198,9 +203,7 @@ mkEntityDef ps name entattribs lines =
             Nothing -> idName ts
             Just s -> s
     uniqs = mapMaybe (takeUniqs ps cols) attribs
-    derives = case mapMaybe takeDerives attribs of
-                [] -> ["Show", "Read", "Eq"]
-                x -> concat x
+    derives = concat $ mapMaybe takeDerives attribs
 
     cols :: [FieldDef]
     cols = mapMaybe (takeCols ps) attribs
@@ -224,17 +227,18 @@ takeCols ps (n:typ:rest)
             Nothing -> error $ "Invalid field type: " ++ show typ
             Just ft -> Just $ FieldDef
                 (HaskellName n)
-                (DBName $ db rest)
+                (DBName $ getDbName ps n rest)
                 ft
                 rest
-  where
-    db [] = psToDBName ps n
-    db (a:as) =
-        case T.stripPrefix "sql=" a of
-            Nothing -> db as
-            Just s -> s
 takeCols _ _ = Nothing
 
+getDbName :: PersistSettings -> Text -> [Text] -> Text
+getDbName ps n [] = psToDBName ps n
+getDbName ps n (a:as) =
+    case T.stripPrefix "sql=" a of
+      Nothing -> getDbName ps n as
+      Just s  -> s
+
 takeUniqs :: PersistSettings
           -> [FieldDef]
           -> [Text]
